﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace InkApp
{
    public partial class Page : UserControl
    {
        private Stroke currentStroke;

        public Page()
        {
            InitializeComponent();
        }

        private void ink_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ink.CaptureMouse();
            currentStroke = new Stroke();
            currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(ink));
            currentStroke.DrawingAttributes.Color = Colors.Black;
            currentStroke.DrawingAttributes.Width = 2;
            currentStroke.DrawingAttributes.Height = 2;
            ink.Strokes.Add(currentStroke);
        }

        private void ink_MouseMove(object sender, MouseEventArgs e)
        {
            if (currentStroke == null) return;
            currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(ink));
        }

        private void ink_LostMouseCapture(object sender, MouseEventArgs e)
        {
            if (currentStroke == null) return;
            StylusPointCollection styles = currentStroke.StylusPoints;
            currentStroke = null;
            Storyboard story = this.Resources["story"] as Storyboard;
            story.Stop();

            DoubleAnimationUsingKeyFrames lefts = story.Children[0] as DoubleAnimationUsingKeyFrames;
            DoubleAnimationUsingKeyFrames tops = story.Children[1] as DoubleAnimationUsingKeyFrames;
            DoubleAnimationUsingKeyFrames angles = story.Children[2] as DoubleAnimationUsingKeyFrames;

            lefts.KeyFrames.Clear();
            tops.KeyFrames.Clear();
            angles.KeyFrames.Clear();

            Frame[] frames = new Frame[styles.Count];
            for (int i = 0; i < styles.Count; i++)
            {
                Frame f = new Frame();
                frames[i] = f;
                f.X = styles[i].X;
                f.Y = styles[i].Y;
                if (i == 0) continue;

                Frame prev = frames[i - 1];
                double dx = f.X - prev.X;
                double dy = f.Y - prev.Y;
                double dl = Math.Sqrt(dx * dx + dy * dy);
                f.Position = prev.Position + dl;
                double rad = Math.Atan2(dy, dx);
                double deg = 180.0 * rad / Math.PI;
                f.Angle = deg;
            }

            long ticksPerPixel = TimeSpan.FromMilliseconds(5).Ticks;
            foreach (Frame f in frames)
            {
                long ticks = (long)((double)ticksPerPixel * f.Position);
                KeyTime key = KeyTime.FromTimeSpan(TimeSpan.FromTicks(ticks));
                lefts.KeyFrames.Add(new LinearDoubleKeyFrame { Value = f.X, KeyTime = key });
                tops.KeyFrames.Add(new LinearDoubleKeyFrame { Value = f.Y, KeyTime = key });
                angles.KeyFrames.Add(new DiscreteDoubleKeyFrame { Value = f.Angle, KeyTime = key });
            }

            story.Begin();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ink.Strokes.Clear();
        }
    }
}
